/*
  +----------------------------------------------------------------------+
  | PHP Version 4                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3 of the PHP license,         |
  | that is bundled with this package in the file LICENSE, and is        |
  | available at through the world-wide-web at                           |
  | http://www.php.net/license/3.txt.                                    |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Authors:    David Oury <doury@php.net>                               |
  +----------------------------------------------------------------------+

  $Id: binaryDataBuffer.c,v 1.1 2007/03/27 14:32:37 doury Exp $ 
*/


/* Standard libraries		*/
#include <stdlib.h>
#include <string.h>
/* Zend API library			*/
#include "php.h"					/* PHP library				*/
/* IMS extension libraries	*/
#include "binaryDataBuffer.h"		/* Binary data buffering	*/


/**
 * New Buffer memory allocation
 */
binaryDataBuffer_t *binaryDataBuffer_new()
{
	binaryDataBuffer_t *	dataBuffer;
		
	if ((dataBuffer = (binaryDataBuffer_t *) emalloc(sizeof(binaryDataBuffer_t))) == NULL)
	{
		return NULL;
	}
	dataBuffer->dataBuffer	= NULL;
	dataBuffer->dataLength	= 0;
	dataBuffer->dataOffset	= 0;
	
	return dataBuffer;
}
/**
 * Buffer initialization
 */
char *binaryDataBuffer_initBuffer(binaryDataBuffer_t *dataBuffer, int length)
{
	/* Memory freeing if needed	*/
	if (dataBuffer->dataBuffer)
	{
		binaryDataBuffer_freeBuffer(dataBuffer);
	}
	/* Memory allocation		*/
	if ((dataBuffer->dataBuffer = (char *) emalloc(length)) == NULL)
	{
		return NULL;
	}
	else
	{
		dataBuffer->dataLength	= length;
		dataBuffer->dataOffset	= 0;
		return dataBuffer->dataBuffer;
	}
}
/**
 * Buffer memory freeing
 */
void binaryDataBuffer_freeBuffer(binaryDataBuffer_t *dataBuffer)
{
	dataBuffer->dataBuffer	= NULL;
	dataBuffer->dataLength	= 0;
	dataBuffer->dataOffset	= 0;
}
/**
 * Buffer internal offset move
 */
int binaryDataBuffer_setOffset(binaryDataBuffer_t *dataBuffer, int offset)
{
	if (offset < 0)
	{
		dataBuffer->dataOffset = 0;
	}
	else if (offset > dataBuffer->dataLength)
	{
		dataBuffer->dataOffset = dataBuffer->dataLength;
	}
	else
	{
		dataBuffer->dataOffset = offset;
	}
}
/**
 * Buffer fill
 */
char *	binaryDataBuffer_setBuffer(binaryDataBuffer_t *dataBuffer, char *src, int srcLen)
{
	/* Buffer initialization	*/
	if (binaryDataBuffer_initBuffer(dataBuffer, srcLen) == NULL)
	{
		return NULL;
	}
	/* Source data memory copy			*/
	memcpy(dataBuffer->dataBuffer, src, srcLen);
	
	return dataBuffer->dataBuffer;
}
/**
 * Buffer data concatenation
 */
char *	binaryDataBuffer_addBuffer(binaryDataBuffer_t *dataBuffer, char *src, int srcLen)
{
	/* If buffer filled -> Concatenation	*/
	if (dataBuffer->dataBuffer)
	{
		char *	newBuffer	= NULL;
		int		newLength	= dataBuffer->dataLength + srcLen;
		
		/* Data buffer memory allocation	*/
		if ((newBuffer = (char *) emalloc(newLength)) == NULL)
		{
			return NULL;
		}
		else
		{
			/* Original data memory copy	*/
			memcpy(newBuffer, dataBuffer->dataBuffer, dataBuffer->dataLength);
			/* Source data memory copy		*/
			memcpy(newBuffer + dataBuffer->dataLength, src, srcLen);
			/* Properties update			*/
			dataBuffer->dataLength	= newLength;
			dataBuffer->dataBuffer	= newBuffer;
		}
		return dataBuffer->dataBuffer;
	}
	/* If buffer empty -> Filling	*/
	return binaryDataBuffer_setBuffer(dataBuffer, src, srcLen);
}
/**
 * Recovery of buffer data to current offset
 */
char *	binaryDataBuffer_getBuffer(binaryDataBuffer_t *dataBuffer)
{
	return ( dataBuffer->dataBuffer + dataBuffer->dataOffset );
}
/**
 * Buffer BYTE writing
 * 
 * 8 first bytes of value written in "big endian"
 */
int binaryDataBuffer_writeByte(binaryDataBuffer_t *dataBuffer, int data)
{
	if (dataBuffer->dataOffset < dataBuffer->dataLength)
	{
		dataBuffer->dataBuffer[dataBuffer->dataOffset++] = (0xff & data);
		return 1;
	}
	else
	{
		return 0;
	}
}
/**
 * Buffer SHORT writing
 * 
 * 16 first bytes of value written in "big endian"
 */
int binaryDataBuffer_writeShort(binaryDataBuffer_t *dataBuffer, int data)
{
	if (dataBuffer->dataOffset < (dataBuffer->dataLength - 1))
	{
		dataBuffer->dataBuffer[dataBuffer->dataOffset++] = (0xff & (data >> 8));
		dataBuffer->dataBuffer[dataBuffer->dataOffset++] = (0xff & data);
		return 2;
	}
	else
	{
		return 0;
	}
}
/**
 * Buffer INTEGER writing
 * 
 * 32 first bytes of value written in "big endian"
 * 
 */
int binaryDataBuffer_writeInt(binaryDataBuffer_t *dataBuffer, int data)
{
	if (dataBuffer->dataOffset < (dataBuffer->dataLength - 3))
	{
		dataBuffer->dataBuffer[dataBuffer->dataOffset++] = (0xff & (data >> 24));
		dataBuffer->dataBuffer[dataBuffer->dataOffset++] = (0xff & (data >> 16));
		dataBuffer->dataBuffer[dataBuffer->dataOffset++] = (0xff & (data >> 8));
		dataBuffer->dataBuffer[dataBuffer->dataOffset++] = (0xff & data);
		return 4;
	}
	else
	{
		return 0;
	}
}
/**
 * Buffer N-BYTES writing
 */
int binaryDataBuffer_writeBytes(binaryDataBuffer_t *dataBuffer, char *data, int data_len)
{
	int	i;
	
	for (i = 0; i < data_len && dataBuffer->dataOffset < dataBuffer->dataLength; i++)
	{
		dataBuffer->dataBuffer[dataBuffer->dataOffset++] = (0xff & data[i]);
	}
	return i;
}
/**
 * Buffer N-BYTES writing + encoding
 */
int binaryDataBuffer_encodeBytes(binaryDataBuffer_t *dataBuffer, char *data, int data_len, char *encoder)
{
	int	i;
	
	if (encoder == NULL)
	{
		return binaryDataBuffer_writeBytes(dataBuffer, data, data_len);
	}
	for (i = 0; i < data_len && dataBuffer->dataOffset < dataBuffer->dataLength; i++)
	{
		dataBuffer->dataBuffer[dataBuffer->dataOffset++] = encoder[0xff & data[i]];
	}
	return i;
}
/**
 * Buffer BYTE reading
 */
int binaryDataBuffer_readByte(binaryDataBuffer_t *dataBuffer, char *data)
{
	*data	= 0x00;
	
	if (dataBuffer->dataOffset < dataBuffer->dataLength)
	{
		*data = dataBuffer->dataBuffer[dataBuffer->dataOffset++];
		return 1;
	}
	else
	{
		return 0;
	}
}
/**
 * Buffer SHORT reading
 * 
 * 16 first bytes of value read in "big endian"
 */
int binaryDataBuffer_readShort(binaryDataBuffer_t *dataBuffer, short *data)
{
	*data	= 0x0000;

	if (dataBuffer->dataOffset < (dataBuffer->dataLength - 1))
	{
		*data |= ((short)dataBuffer->dataBuffer[dataBuffer->dataOffset++]) << 8;
		*data |= ((short)dataBuffer->dataBuffer[dataBuffer->dataOffset++]);
		return 2;
	}
	else
	{
		return 0;
	}
}
/**
 * Buffer INTEGER reading
 * 
 * 32 first bytes of value read in "big endian"
 * 
 */
int binaryDataBuffer_readInt(binaryDataBuffer_t *dataBuffer, int *data)
{
	*data	= 0x00000000;

	if (dataBuffer->dataOffset < (dataBuffer->dataLength - 3))
	{
		*data |= (int)(dataBuffer->dataBuffer[dataBuffer->dataOffset++]) << 24;
		*data |= (int)(dataBuffer->dataBuffer[dataBuffer->dataOffset++]) << 16;
		*data |= (int)(dataBuffer->dataBuffer[dataBuffer->dataOffset++]) << 8;
		*data |= (int)(dataBuffer->dataBuffer[dataBuffer->dataOffset++]);
		return 4;
	}
	else
	{
		return 0;
	}
}
/**
 * Buffer N-BYTES reading
 */
int binaryDataBuffer_readBytes(binaryDataBuffer_t *dataBuffer, char *data, int data_len)
{
	int	i;
	
	for (i = 0; i < data_len && dataBuffer->dataOffset < dataBuffer->dataLength; i++)
	{
		data[i] = dataBuffer->dataBuffer[dataBuffer->dataOffset++] & 0xff;
	}
	return i;
}
/**
 * Buffer N-BYTES reading + decoding
 */
int binaryDataBuffer_decodeBytes(binaryDataBuffer_t *dataBuffer, char *data, int data_len, char *decoder)
{
	int	i;
	
	if (decoder == NULL)
	{
		return binaryDataBuffer_readBytes(dataBuffer, data, data_len);
	}
	for (i = 0; i < data_len && dataBuffer->dataOffset < dataBuffer->dataLength; i++)
	{
		data[i] = decoder[0xff & dataBuffer->dataBuffer[dataBuffer->dataOffset++]];
	}
	return i;
}
