/*
  +----------------------------------------------------------------------+
  | PHP Version 4                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3 of the PHP license,         |
  | that is bundled with this package in the file LICENSE, and is        |
  | available at through the world-wide-web at                           |
  | http://www.php.net/license/3.txt.                                    |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Authors:    David Oury <doury@php.net>                               |
  +----------------------------------------------------------------------+

  $Id: msg_wrap.c,v 1.1 2007/03/27 14:32:37 doury Exp $ 
*/


/* Standard libraries		*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
/* Zend API library			*/
#include "php.h"					/* PHP : Zend API			*/
/* IMS extension libraries	*/
#include "common.h"					/* Common					*/
#include "transcoder.h"				/* IMS Transcoding			*/
#include "string.h"					/* IMS Strings				*/
#include "field.h"					/* IMS Data fields			*/
#include "xml.h"					/* XML Processing			*/
#include "msg_wrap.h"				/* XML Message Wrapping		*/
#include "log.h"					/* Log & Debug				*/


/**
 * XML file parsing
 * IMS message XML Wrapper setting
 */
int MSG_parseXML(const char *filename, MSG_wrapper_t *wrapper, TRANS_trans_t *trans)
{
	IMS_xmlData_t	xml_data;		/* XML parsing data			*/
	int				result;			/* Function return code		*/
	
debug("MSG_parseXML() -> START");
	/* XML Wrapper initialization	*/
	if (wrapper == NULL && (wrapper = MSG_newWrapper(trans)) != IMS_CODE_OK) {
		return IMS_CODE_ERR_MALLOC;
	} else {
		MSG_initWrapper(wrapper, trans);
	}
	/* XML parsing config settings	*/
	IMS_xmlConf_t config[]	= {	IMS_xml_config("message", MSG_startElem_Message, NULL),
								IMS_xml_config("field", MSG_startElem_Field, NULL),
								IMS_xml_config(NULL, NULL, NULL)
							};
	/* XML data loading and init	*/
	IMS_xml_initData(&xml_data, filename, wrapper, config);
	/* XML file parsing				*/
	result = IMS_xml_parse(&xml_data);
	
debug("MSG_parseXML() -> END");
	return result;
}

/**
 * IMS message Wrapper memory allocation
 */
MSG_wrapper_t *MSG_newWrapper(TRANS_trans_t *trans)
{
	MSG_wrapper_t *	wrapper;		/* Formateur XML de message	*/
	
	if (EMALLOC_IS_NULL(wrapper, MSG_wrapper_t)) {
		return NULL;
	} else {
		MSG_initWrapper(wrapper, trans);
		return wrapper;
	}
}

/**
 * IMS message Wrapper memory freeing
 */
int	MSG_freeWrapper(MSG_wrapper_t *wrapper)
{
	free(wrapper);
}

/**
 *  IMS message Wrapper initialization
 */
int	MSG_initWrapper(MSG_wrapper_t *wrapper, TRANS_trans_t *trans)
{
	/* Transcoding selection if empty	*/
	if (trans == NULL)
		trans = TRANS_select(NULL);
	
	wrapper->length		= 0;			/* Message length			*/
	wrapper->encoding	= NULL;			/* Encoding code			*/
	wrapper->trans		= trans;		/* Selected transcoding		*/
	wrapper->f_list		= NULL;			/* IMS Data fields list		*/
	wrapper->f_last		= NULL;			/* Last data field in list	*/
	wrapper->f_size		= 0;			/* Chained fields list size	*/

	return IMS_CODE_OK;
}

/**
 * IMS message Wrapper transcoding selection
 */
int	MSG_setTranscoder(MSG_wrapper_t *wrapper, const char *encoding)
{
	TRANS_trans_t *	trans;				/* Selected transcoding		*/

	if (trans = TRANS_select(encoding)) {
		wrapper->trans		= trans;
		wrapper->encoding	= trans->name;
		return IMS_CODE_OK;
	} else {
		return IMS_CODE_ERR_ENCODING;
	}
}

/**
 * XML Wrapper <MESSAGE> starting tag handler
 */
int	MSG_startElem_Message(IMS_xmlData_t *xml_data)
{
	IMS_xmlAttr_t	attribute;			/* Element attribute		*/
	
	/* XML Wrapper casting	*/
	MSG_CAST_WRAPPER(wrapper, xml_data);
	/* Attributes parsing	*/
	while (IMS_xml_nextAttribute(&attribute, &(xml_data->atts))) {
		if (STR_IS_EQUAL(attribute.name, "encoding")) {
			MSG_setTranscoder(wrapper, attribute.value);
		}
	}
}

/**
 * XML Wrapper <MESSAGE> ending tag handler
 */
int	MSG_endElem_Message(IMS_xmlData_t *xml_data)
{
	/* XML Wrapper casting	*/
	MSG_CAST_WRAPPER(wrapper, xml_data);
}

/**
 * XML Wrapper <FIELD> starting tag handler -> IMS Data Field creation
 */
int	MSG_startElem_Field(IMS_xmlData_t *xml_data)
{
	IMS_field_t *	field;				/* IMS data field to parse	*/
	IMS_xmlAttr_t	attribute;			/* Current XML attribute	*/
	char *			source;				/* String source value		*/
	int				result;				/* Function return code		*/
	
debug("MSG_startElem_Field() -> START");
	source	= NULL;
	/* XML Wrapper casting			*/
	MSG_CAST_WRAPPER(wrapper, xml_data);
	/* IMS data field creation		*/
	if ((field = IMS_newField()) == NULL) {
		return IMS_CODE_ERR_MALLOC;
	} else {
		/* Field type set to STRING				*/
		IMS_setFieldType(field, IMS_TYPE_STRING);
		/* Transcoding set to Wrapper default	*/
		FIELD_VSTR(field).trans	= wrapper->trans;
	}
	/* Field attributes parsing		*/
debug("MSG_startElem_Field() -> Attributes parsing");
	while (IMS_xml_nextAttribute(&attribute, &(xml_data->atts))) {
		if ((result = MSG_setFieldAttr(field, &attribute, &source)) != IMS_CODE_OK) {
			return result;
		}
	}
	/* Field type & value settings	*/
debug("MSG_startElem_Field() -> Value settings");
	if (source && (result = IMS_setField_str(field, source, strlen(source))) != IMS_CODE_OK) {
		return result;
	}
	/* Field insertion in list		*/
	if ((field = IMS_insertField(field, &(wrapper->f_list))) == NULL) {
		return IMS_CODE_ERR_FIELD;
	} else {
		wrapper->length	+= field->length;
		wrapper->f_last	= field;
		wrapper->f_size++;
	}
debug("MSG_startElem_Field() -> END");
	return IMS_CODE_OK;
}

/**
 * IMS data field attribute setting
 */
int MSG_setFieldAttr(IMS_field_t *field, IMS_xmlAttr_t *attribute, char **source)
{
	TRANS_trans_t *	trans;		/* Field transcoding	*/
	
debug("MSG_setFieldAttr() -> START");

	if (attribute->name == NULL || attribute->value == NULL) {
		return IMS_CODE_ERR_ATTRIBUTE;
	} /* Field name				*/
	else if (STR_IS_EQUAL(attribute->name, "name")) {
		if ((field->name = (char *) emalloc(strlen(attribute->value) + 1)) == NULL)
			return IMS_CODE_ERR_MALLOC;
		strcpy(field->name, attribute->value);
	} /* Field length			*/
	else if (STR_IS_EQUAL(attribute->name, "length")) {
		field->length = atoi(attribute->value);
	} /* LLZZ segment separator	*/
	else if (STR_IS_EQUAL(attribute->name, "seginit")) {
		if (field->segInit = STR_IS_EQUAL(attribute->value, "true"))
			IMS_setFieldType(field, IMS_TYPE_INT);
	} /* Field attribute		*/
	else if (STR_IS_EQUAL(attribute->name, "attribute")) {
		field->attribute = STR_IS_EQUAL(attribute->value, "true");
	} /* Field filler -> 1st valid char or binary zero	*/
	else if (STR_IS_EQUAL(attribute->name, "filler")) {
		FIELD_VSTR(field).filler = strlen(attribute->value) ? (attribute->value)[0] : 0x00;
	} /* Field alignement		*/
	else if (STR_IS_EQUAL(attribute->name, "align")) {
		if (STR_IS_EQUAL(attribute->value, "LEFT")) {
			FIELD_VSTR(field).align = IMS_ALIGN_LEFT;
		} else if (STR_IS_EQUAL(attribute->value, "RIGHT")) {
			FIELD_VSTR(field).align = IMS_ALIGN_RIGHT;
		} else if (STR_IS_EQUAL(attribute->value, "CENTER")) {
			FIELD_VSTR(field).align = IMS_ALIGN_CENTER;
		} else if (STR_IS_EQUAL(attribute->value, "DEFAULT")) {
			FIELD_VSTR(field).align = IMS_ALIGN_DEFAULT;
		} else {
			return IMS_CODE_ERR_ATTRIBUTE;
		}
	} /* Default value			*/
	else if (STR_IS_EQUAL(attribute->name, "default")) {
		*source	= attribute->value;
	} /* Field transcoding		*/
	else if (STR_IS_EQUAL(attribute->name, "encoding")) {
		if (TRANS_getByName(&trans, attribute->value) == TRANS_RESULT_OK) {
			FIELD_VSTR(field).trans = trans;
		} else {
			return IMS_CODE_ERR_ENCODING;
		}
	}
/* TODO								*/
/* LOG display to remove			*/
char	buf[400];
snprintf(buf, sizeof(buf), "%s = %s", attribute->name, attribute->value);
debug(buf);
/************************************/
debug("MSG_setFieldAttr() -> END");

	return IMS_CODE_OK;
}

/**
 * XML Wrapper <FIELD> ending tag handler -> IMS Data Field closure
 */
int	MSG_endElem_Field(IMS_xmlData_t *xml_data)
{
	/* XML Wrapper casting			*/
	MSG_CAST_WRAPPER(wrapper, xml_data);
}
