/*
  +----------------------------------------------------------------------+
  | PHP Version 4                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3 of the PHP license,         |
  | that is bundled with this package in the file LICENSE, and is        |
  | available at through the world-wide-web at                           |
  | http://www.php.net/license/3.txt.                                    |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Authors:    David Oury <doury@php.net>                               |
  +----------------------------------------------------------------------+

  $Id: string.c,v 1.1 2007/03/27 14:32:37 doury Exp $ 
*/


/* Standard libraries		*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
/* Zend API library			*/
#include "php.h"						/* PHP : Zend API			*
/* IMS extension libraries	*/
#include "common.h"						/* Common					*/
#include "transcoder.h"					/* IMS Transcoding			*/
#include "string.h"						/* IMS Strings				*/
#include "log.h"						/* Log & Debug				*/


/**
 * IMS String memory allocation
 */
int IMS_newString(IMS_str_t **str, const char *src, int srcLen, char filler, char align, TRANS_trans_t *trans)
{
	int	result	= IMS_CODE_OK;			/* Function return code		*/
	
	if (EMALLOC_IS_NULL(*str, IMS_str_t)) {
		return IMS_CODE_ERR_MALLOC;
	}
	(*str)->string	= NULL;
	(*str)->length	= 0;

	if ((result = IMS_initString(*str, src, srcLen, filler, align, trans)) != IMS_CODE_OK) {
		IMS_freeString(*str);
		return result;
	}
	return IMS_CODE_OK;
}

/**
 * IMS String initialization
 */
int	IMS_initString(IMS_str_t *str, const char *src, int srcLen, char filler, char align, TRANS_trans_t *trans)
{
	int	result	= IMS_CODE_OK;			/* Function return code		*/
	
	/* String settings		*/
	if ((result = IMS_setString(str, src, srcLen)) != IMS_CODE_OK) {
		return result;
	}
	/* Properties settings	*/
	str->filler	= filler;
	switch (align)
	{
		case IMS_ALIGN_LEFT :			/* Alignement : LEFT		*/
		case IMS_ALIGN_RIGHT :			/* Alignement : RIGHT		*/
		case IMS_ALIGN_CENTER :			/* Alignement : CENTER		*/
			str->align = align;
			break;
		default :
			str->align = IMS_ALIGN_DEFAULT;
	}
	str->trans	= trans ? trans : TRANS_select(NULL);
	
	return IMS_CODE_OK;	
}

/**
 * IMS String settings
 */
int IMS_setString(IMS_str_t *str, const char *src, int srcLen)
{
	str->string	= NULL;
	str->length	= 0;
	/* Source data assignation	*/
	if (src && srcLen) {
		if ((str->string = (char *) emalloc(srcLen)) == NULL) {
			return IMS_CODE_ERR_MALLOC;
		} else {
			str->string	= memcpy(str->string, src, srcLen);
			str->length	= srcLen;
		}
	}
	return IMS_CODE_OK;
}

/**
 * IMS String memory freeing
 */
int IMS_freeString(IMS_str_t *str)
{
	efree(str->string);
	
	return IMS_CODE_OK;
}

/**
 * IMS String writing
 */
int IMS_writeString(IMS_str_t *str, char **buf, int length)
{
	int	space;							/* Total padding			*/
	int	spaceLeft;						/* Left padding				*/
	int	spaceRight;						/* Light padding			*/
	int	posStart;						/* Starting position 		*/
	int	posEnd;							/* Ending position			*/
	int pos, index;						/* Position & buffer index	*/
	
	/* Memory allocation if no buffer	*/
	if (*buf == NULL && (*buf = (char *)emalloc(length)) == NULL) {
		return IMS_CODE_ERR_MALLOC;
	}
	/* Positions & tabs processings		*/
	if ((space = length - str->length) < 0) {
		space		= -space;
		spaceLeft	= 0;
		spaceRight	= 0;
		switch (str->align) {
			case IMS_ALIGN_LEFT :			/* Align LEFT	*/
				posStart	= 0;
				posEnd		= length;
				break;
			case IMS_ALIGN_RIGHT :			/* Align RIGHT	*/
				posStart	= space;
				posEnd		= str->length;
				break;
			case IMS_ALIGN_CENTER :			/* Align CENTER	*/
				posStart	= (space / 2);
				posEnd		= str->length - (space / 2) - (space % 2);
				break;
		}
	} else {
		posStart	= 0;
		posEnd		= str->length;
		switch (str->align) {
			case IMS_ALIGN_LEFT :			/* Align LEFT	*/
				spaceLeft	= 0;
				spaceRight	= space;
				break;
			case IMS_ALIGN_RIGHT :			/* Align RIGHT	*/
				spaceLeft	= space;
				spaceRight	= 0;
				break;
			case IMS_ALIGN_CENTER :			/* Align CENTER	*/
				spaceLeft	= (space / 2);
				spaceRight	= (space / 2) + (space % 2);
				break;
		}
	}
	/* Writing into buffer		*/
	index = 0;
	while (index < length && spaceLeft--)						/* Left padding		*/
		(*buf)[index++]	= str->filler;
	for (pos = posStart; index < length && pos < posEnd; pos++)	/* String copy		*/
		(*buf)[index++]	= (str->string)[pos];
	while (index < length && spaceRight--)						/* Right padding	*/
		(*buf)[index++]	= str->filler;
	/* Buffer encoding			*/
	if (str->trans && TRANS_encode(*buf, length, str->trans) != TRANS_RESULT_OK)
		return IMS_CODE_ERR_TRANSCODE;

	return IMS_CODE_OK;
}

/**
 * IMS String reading
 */
int IMS_readString(IMS_str_t *str, char *buf, int length)
{
	/* Buffer decoding			*/
	if (str->trans && TRANS_decode(buf, length, str->trans) != TRANS_RESULT_OK)
		return IMS_CODE_ERR_TRANSCODE;
	/* IMS string assignation	*/
	return IMS_setString(str, buf, length);
}
