/*
  +----------------------------------------------------------------------+
  | PHP Version 4                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3 of the PHP license,         |
  | that is bundled with this package in the file LICENSE, and is        |
  | available at through the world-wide-web at                           |
  | http://www.php.net/license/3.txt.                                    |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Authors:    David Oury <doury@php.net>                               |
  +----------------------------------------------------------------------+

  $Id: transcoder.h,v 1.1 2007/03/27 14:32:37 doury Exp $ 
*/


/**
 *  NOTES about this extension, its loading and transcoding processing :
 * 
 *	- Global transcoding context initialization					[TRANS_ref_t]
 * 
 *	- Transcoding context includes :
 *						->	the list of translater builders		[TRANS_list_t]
 *						->	the array of created translaters	[TRANS_trans_t]
 *						->	the builders/translaters counter	[int count]
 * 
 *	- Transcoding context is impacted by transcoding source files :
 *						->	every source defines a new translater builder
 *						->	this builder is inserted into context list
 *						->	the counter is incremented
 *						->	defined macros allow encapsulation of builders as
 * 							conversion tables alone
 * 
 *	- As the builders list is processed :						[TRANS_list_t]
 *						->	Builder function execution			[TRANS_build_f]
 *						->	New translater creation				[TRANS_trans_t]
 *						->	Translater insertion into array		
 *						->	Translaters counter update			[int count]
 * 
 *	- Every translater includes :
 *						->	an auto-generated index				[int]
 *						->	a name for searching/selection		[char *]
 *						->	an encoding conversion table		[char *]
 *						->	an decoding conversion table		[char *]
 * 
 *	- Translater searching and selection :
 *						->	searching executes an iterative name test
 *						->	selection returns the translater pointer if found
 */

#ifndef	_IMS_TRANSCODER_H_
#define	_IMS_TRANSCODER_H_

/*********************************************************************
 * Includes
 */

#include "../trans/transcoders.h"			/* Translaters declaration		*/
#include <stdlib.h>

/*********************************************************************
 * Constants definitions
 */

/* Global context name				*/

#define	TRANS_REF_GBL				transcoRef_g

/* Translater names					*/

#define	TRANS_DIRECT_NAME			"DIRECT"
#define	TRANS_DEFAULT_NAME			"DEFAULT"

/* String and array sizes			*/

#define	TRANS_TBL_SIZE				256		/* Conversion tables size		*/
#define	TRANS_ERR_SIZE				200		/* Error messages size			*/

/* Flags							*/

#define	TRANS_TRUE					1		/* Boolean : TRUE				*/
#define	TRANS_FALSE					0		/* Boolean : FALSE				*/
#define	TRANS_ENCODING				1		/* Encoding conversion code		*/
#define	TRANS_DECODING				-1		/* Decoding conversion code		*/

/* Function return codes			*/

#define	TRANS_RESULT_OK				0		/* Return code : OK				*/
#define	TRANS_ERR_SRC_TBL			100		/* Error : Source tables size	*/
#define	TRANS_ERR_MALLOC			200		/* Error : Memory allocation	*/
#define	TRANS_ERR_UNKNOWN			300		/* Error : Unknown translater	*/
#define	TRANS_ERR_TYPE				400		/* Error : Invalid transcoding	*/

/*********************************************************************
 * Data structures declarations
 */

/* Translater builder settings		*/

typedef struct {
	
	char *	name;							/* Translater name				*/
	char *	encoder;						/* Encoding conversion table	*/
	int		enc_size;						/* Decoding table size			*/
	char *	decoder;						/* Encoding conversion table	*/
	int		dec_size;						/* Decoding table size			*/
	char	isDefault;						/* Flag : Default translater	*/
	char	isDirect;						/* Flag : Direct translater		*/

} TRANS_build_t;

/* Translater builder				*/
/* -> Function pointer type			*/

typedef int (*TRANS_builder)();

/* Builders chained list type		*/

typedef struct TRANS_list_s {
	
	TRANS_builder			build;			/* Builder function pointer		*/
	struct TRANS_list_s *	next;			/* Next builder item			*/
	
} TRANS_list_t;	

/* Translater settings				*/

typedef struct {
	
	int		index;							/* Translater index				*/
	char *	name;							/* Translater name				*/
	char	encoder[TRANS_TBL_SIZE];		/* Encoding conversion table	*/
	char	decoder[TRANS_TBL_SIZE];		/* Decoding conversion table	*/

} TRANS_trans_t;

/* Transcoding context				*/

typedef struct {
	
	int					counter;			/* Translaters counter			*/
	TRANS_list_t *		builders;			/* Builders chained list		*/
	TRANS_trans_t *		trans_arr;			/* Translaters array			*/
	TRANS_trans_t *		trans_def;			/* Default translater			*/

} TRANS_ref_t;

/*********************************************************************
 * Translater builders macros delcarations
 *	-> builders writing simplification
 *	-> translaters registery automation
 */

/* Translater builders naming				*/

#define	TRANS_BUILDER(ID)				TRANS_build_##ID

/* Builder declaration for initialization	*/

#define	DECLARE_TRANSCODER(ID)			TRANS_declare(TRANS_BUILDER(ID));	\

/* Default builder definition user macro	*/

#define	DEFAULT_START_TRANSCODER(ID)	_START_TRANSCODER(ID, TRANS_TRUE)

/* Builder definition user macro			*/

#define	START_TRANSCODER(ID)			_START_TRANSCODER(ID, TRANS_FALSE)

/* Builder definition (function beginning)	*/

#define	_START_TRANSCODER(ID, DEFAULT)			\
int TRANS_BUILDER(ID)()							\
{												\
TRANS_log(TRANS_RESULT_OK, "TRANS_build_"#ID"() -> START ", NULL);	\
	TRANS_build_t	building;					\
												\
	building.name		= #ID;				\
TRANS_log(TRANS_RESULT_OK, "TRANS_build_"#ID"() -> name = %s", building.name);	\
	building.isDefault	= DEFAULT;				\
TRANS_log(TRANS_RESULT_OK, "TRANS_build_"#ID"() -> isDefault = %s", (building.isDefault ? "TRUE" : "FALSE"));	\
	building.isDirect	= TRANS_FALSE;			\
TRANS_log(TRANS_RESULT_OK, "TRANS_build_"#ID"() -> isDirect = %s", (building.isDirect ? "TRUE" : "FALSE"));	\

/* Encoding conversion table declaration	*/

#define	SET_ENCODER		char encoder[]	=

/* Decoding conversion table declaration	*/

#define	SET_DECODER		char decoder[]	=

/* Builder definition (function ending)		*/

#define	END_TRANSCODER							\
	building.encoder	= encoder;				\
	building.enc_size	= sizeof(encoder);		\
	building.decoder	= decoder;				\
	building.dec_size	= sizeof(decoder);		\
												\
TRANS_log(TRANS_RESULT_OK, "TRANS_build_ID()    -> END", NULL);	\
	return TRANS_build(&building);				\
}												\

/***********************************************************************************
#define	TRANSCODE_SET(_code)			setTrans__code

#define	TRANSCODE_DEFINE_START(_code)	int setTrans__code(char *encoding, char *decoding) {

#define	TRANSCODE_DEFINE_END(_encoder, _decoder)			\
		int code;											\
		if (sizeof(_encoder) < TRAN_TBL_SIZE ||				\
			sizeof(_decoder) < TRAN_TBL_SIZE)				\
			return TRAN_ERR_SRC_TABLES;						\
			encoding[code] = _encoder[code];				\
			decoding[code] = _decoder[code];				\
		}													\
		return TRAN_RESULT_OK;								\
	}														\

/*********************************************************************
 * Functions prototypes & macros declarations
 */

/* Shortcut macros							*/

#define	ENCODER								TRANS_encode
#define	DECODER								TRANS_decode
#define	SELECT(_name)						TRANS_select(_name)
#define	ENCODE(_str, _len, _trans)			ENCODER(_str, _len, _trans, TRANS_ENCODING)
#define	DECODE(_str, _len, _trans)			DECODER(_str, _len, _trans, TRANS_DECODING)
#define	ENCODE_SELECT(_str, _len, _name)	ENCODER(_str, _len, SELECT(_name))
#define	DECODE_SELECT(_str, _len, _name)	DECODER(_str, _len, SELECT(_name))

/* Transcoding context management			*/

int	TRANS_reset();
int	TRANS_init();
int	TRANS_free();
int TRANS_declare(TRANS_builder builder);
int TRANS_insert_builder(TRANS_list_t **builders, TRANS_builder builder);
int TRANS_delete_builders(TRANS_list_t **builders);
int TRANS_build(TRANS_build_t *building);
int TRANS_phpLog(int error, const char *format, const char *detail);
int TRANS_log(int error, const char *format, const char *detail);

/* Translater selection and transcoding processing	*/

TRANS_trans_t *	TRANS_select(const char *name);

int	TRANS_getByIndex(TRANS_trans_t **trans, int index);
int	TRANS_getByName	(TRANS_trans_t **trans, const char *name);

int TRANS_encode	(char *str, int len, TRANS_trans_t *trans);
int TRANS_decode	(char *str, int len, TRANS_trans_t *trans);
int	TRANS_transcode	(char *str, int len, TRANS_trans_t *trans, char type);

/* Functions and macros not in use			*/

#define	GET_TRANS_REF	&TRANS_REF_GBL

TRANS_ref_t *			TRANS_get_ref();

int	TRANS_build_CP297();

#endif
