/*
  +----------------------------------------------------------------------+
  | PHP Version 4                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2004 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3 of the PHP license,         |
  | that is bundled with this package in the file LICENSE, and is        |
  | available at through the world-wide-web at                           |
  | http://www.php.net/license/3.txt.                                    |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Authors:    David Oury <doury@php.net>                               |
  +----------------------------------------------------------------------+

  $Id: xml.c,v 1.1 2007/03/27 14:32:37 doury Exp $ 
*/


/* Standard libraries		*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
/* Zend API library			*/
#include "php.h"					/* PHP : Zend API			*/
/* IMS extension libraries	*/
#include "common.h"					/* Common					*/
#include "xml.h"					/* XML Processing			*/
#include "log.h"					/* Log & Debug				*/


/**
 * XML parsing parameters settings
 */
IMS_xmlConf_t IMS_xml_config(char *tag, IMS_xml_handler start, IMS_xml_handler end)
{
	IMS_xmlConf_t	config;			/* XML parsing confi		*/
	
	config.tag		= tag;
	config.start	= start;
	config.end		= end;
	
	return config;
}

/**
 * XML parsing data initialization
 */
int	IMS_xml_initData(IMS_xmlData_t *xml_data, const char *filename, void *wrapper, IMS_xmlConf_t *config)
{
	xml_data->filename	= (char *)filename;	/* File name to parse		*/
	xml_data->depth		= 0;				/* XML parsing depth		*/
	xml_data->tag		= NULL;				/* Current XML element tag	*/
	xml_data->atts		= NULL;				/* Current XML attributes	*/
	xml_data->wrapper	= wrapper;			/* XML message Wrapper		*/
	xml_data->config	= config;			/* XML parsing config		*/

	return IMS_CODE_OK;
}

/**
 * XML parsing initialization (EXPAT standard library)
 */
int	IMS_xml_initParser(IMS_xmlData_t *xml_data, XML_Parser *xmlParser)
{
	debug("IMS_xml_initParser() -> START");

	/* XML parser creation				*/
	if ((*xmlParser = XML_ParserCreate(NULL)) == NULL) {
		debug("IMS_xml_initParser() -> Erreur : XML_ParserCreate()");
		return IMS_CODE_ERR_XML_INIT;
	}
	/* XML parsing data settings		*/
	XML_SetUserData(*xmlParser, xml_data);
	/* XML elements handlers settings	*/
	XML_SetElementHandler(*xmlParser, IMS_xml_startElement, IMS_xml_endElement);
	/* XML strings handlers settings	*/
	XML_SetCharacterDataHandler(*xmlParser, IMS_xml_charData);
	/* XML CDATA handlers settings		*/

/*	2006/11/29 - d_ou - Disabled for PHP 5 compatibility

	XML_SetCdataSectionHandler(*xmlParser, IMS_xml_startCData, IMS_xml_endCData);
*/
	debug("IMS_xml_initParser() -> END");

	return IMS_CODE_OK;
}

/**
 * XML parsing execution (EXPAT standard library)
 */
int IMS_xml_parse(IMS_xmlData_t *xml_data)
{
	XML_Parser	xmlParser;				/* XML parser				*/
	FILE *		file;					/* File name to parse		*/
	char		buf[XML_BUFFER_SIZE];	/* File reading buffer		*/
	int			length;					/* Read data length			*/
	int			done;					/* Parsing stop flag		*/
	int			result = IMS_CODE_OK;	/* Function return code		*/

debug("IMS_xml_parse() -> START");
		
	if ((file = fopen(xml_data->filename, "r")) == NULL) {
		debug("IMS_xml_parse() -> File error");
		return XML_CODE_ERR_FILE;
	}
	if ((result = IMS_xml_initParser(xml_data, &xmlParser)) != IMS_CODE_OK) {
		debug("IMS_xml_parse() -> Initialization error");
		return result;
	}
	do {
		length	= fread(buf, sizeof(char), sizeof(buf), file);
		done	= length < sizeof(buf);
		if (!XML_Parse(xmlParser, buf, length, done)) {
			snprintf(	buf, sizeof(buf), "%s at line %d",
						XML_ErrorString(XML_GetErrorCode(xmlParser)),
						XML_GetCurrentLineNumber(xmlParser)
					);
			debug(buf);
			result	= IMS_CODE_ERR_XML_PARSE;
			done	= IMS_FALSE;
		}
	} while (!done);
	
	XML_ParserFree(xmlParser);

debug("IMS_xml_parse() -> END");
	return result;
}

/**
 * XML element opening tag processing (EXPAT standard library)
 */
void IMS_xml_startElement(void *userData, const char *name, const char **atts)
{
	IMS_xmlConf_t *	config;				/* XML parsing config		*/
	char			buf[200];

	snprintf(buf, 200, "IMS_xml_startElement() -> TAG <%s>", name);
	debug(buf);

	/* XML parsing data		*/
	IMS_CAST_XML(xml_data, userData);
	/* XML data settings	*/
	xml_data->depth++;
	xml_data->tag	= (char *)	name;
	xml_data->atts	= (char **)	atts;
	/* Element processing	*/
	config = xml_data->config;
	while (config->tag) {
		if (STR_IS_EQUAL(config->tag, name)) {
			config->start(xml_data);
			break;
		}
		config++;
	}
}

/**
 * Next XML element attribute fetching
 *	ps_attribute	-> structure pointer on IMS_xmlAttr_t
 *	pa_attributes	-> array pointer on attributes strings
 */
int IMS_xml_nextAttribute(IMS_xmlAttr_t *ps_attribute, char ***pa_attributes)
{
	if (ps_attribute && pa_attributes && *pa_attributes && **pa_attributes) {
		ps_attribute -> name	= (*pa_attributes)[0];
		ps_attribute -> value	= (*pa_attributes)[1];
		*pa_attributes += 2;
		return IMS_TRUE;
	}
	return IMS_FALSE;
}

/**
 * XML element closing tag processing (EXPAT standard library)
 */
void IMS_xml_endElement(void *userData, const char *name)
{
	char buf[100];
	
	snprintf(buf, 100, "IMS_xml_endElement()    -> TAG </%s>", name);
	debug(buf);
}

/**
 * XML characters data processing (EXPAT standard library)
 */
void IMS_xml_charData(void *userData, const XML_Char *data, int length)
{
}

void IMS_xml_startCData(void *userData)
{
}

void IMS_xml_endCData(void *userData)
{
}
